(** Default implementation which does nothing. 

This module is expected to be used when you only need to implement an analyze
over a limited part of the whole syntax. *)

module type T = sig
  type t

  val default : t
end

module Expression (T' : T) = struct
  (** 
      Describe a variable, using the name in capitalized text, and an optionnal
      index.

      If missing, the index should be considered as [0].
   *)

  type t' = T'.t

  let ident : (S.pos, T'.t) S.variable -> T'.t = fun _ -> T'.default

  (*
        Basic values, text, number…
   *)

  let integer : S.pos -> string -> T'.t = fun _ _ -> T'.default
  let literal : S.pos -> T'.t T.literal list -> T'.t = fun _ _ -> T'.default

  (** Call a function. The functions list is hardcoded in lib/lexer.mll *)
  let function_ : S.pos -> T.function_ -> T'.t list -> T'.t =
   fun _ _ _ -> T'.default

  (** Unary operator like [-123] or [+'Text']*)
  let uoperator : S.pos -> T.uoperator -> T'.t -> T'.t = fun _ _ _ -> T'.default

  (** Binary operator, for a comparaison, or an operation *)
  let boperator : S.pos -> T.boperator -> T'.t -> T'.t -> T'.t =
   fun _ _ _ _ -> T'.default
end
